# Copyright 2006 HMC Architects, Ontario, CA, USA
#
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided the above
# copyright notice appear in all copies.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#------------------------------------------------------------------------------------------------------------------------
# Name: 		PutOnLayer.rb 
#
# Version: 		v1.00
#
# Date: 		3/17/2006
#
# Author: 		Chris Grant @ HMC Architects
#
# Menu Item: 	Right Click Menu -> Put on Layer:
#
# Description: 	Allows you to easily move entities and their nested elements (ie groups within components within groups) to any layer
#			 	You just right click on an entity you want to move. You'll see a "Put on Layer: " submenu. The first entry is "Current"
#				then all other layers in the file are listed underneath for quickly moving entities / groups / components to different layers.
#
# History:		** Original version was "PutOnCurrentLayer.rb" which always put the selection on the currently active layer. Thanks to Jim H (jimij5) from
#				the SketchUp Ruby forum for the idea to add the ability to put entities on any layer without having to actually change the active layer.
#
# Credit: 		Originally based on "layer.rb" - "change to current layer" by zdarc@163.com. However, other than the include statement this script is entirely custom. 
#				Without layer.rb as a starting place though, I may have never even bothered implementing this...
#				** Thanks wehby for the prompting to checkout Windowizer.rb which uses a dynamic right click menu. That helped me get my head around what I needed to do for this.
#------------------------------------------------------------------------------------------------------------------------
		
require 'sketchup.rb'

def putCurrentRecursion(ents,clayer)
	etype = ents.typename
	case etype
		when "Face"
			ents.layer=clayer if ents.layer!=clayer #if layer needs to be changed		
		when "Edge"
			ents.layer=clayer if ents.layer!=clayer #if layer needs to be changed				
		when "Group"				
			ents.layer=clayer if ents.layer!=clayer
			ents.entities.each do |ent|
				putCurrentRecursion(ent,clayer) #recursion happens here
			end
	 	when "ComponentInstance"
			ents.layer=clayer if ents.layer!=clayer
			begin
				ents.definition.entities.each do |ent|
					putCurrentRecursion(ent,clayer) #recursion happens here
				end
			rescue => err
				puts "EXCEPTION: #{err}"
			end
		else
			begin
				old = ents.layer.name.to_s
				new = clayer.name.to_s
				ents.layer=clayer if ents.layer!=clayer
				puts "Old: " + old + ", New: " + new
			rescue => err
				puts "EXCEPTION: #{err}"
			end
	end
end

def putCurrentLayer(lay)
	model = Sketchup.active_model #this is the current sketchup model environemnt
	layers = model.layers #current set of layers
	current = lay #assign "current" layer as layer passed by right click menu. 
	currentname = current.name.to_s #layer name as string. Though negligible, its faster to reference string rather than layer.name.convert to string every time...
	ents = model.active_entities #current active entities in model
	sel=model.selection #current selection
		
	selindex=1
	model.start_operation ("Put on Layer: " + currentname)
	sel.each do |selEnt|
		Sketchup.set_status_text("Putting " + selindex.to_s + " of " + sel.count.to_s + " entities on " + currentname + "..." + "*"*selindex<<"") #set statusbar to fake progress bar
		putCurrentRecursion(selEnt,current)		
		selindex+=1 #increment progress index
	end
	model.commit_operation #allows for UNDO
	Sketchup.set_status_text("")
end

if( not file_loaded?("PutOnLayer.rb") )	
	UI.add_context_menu_handler do |menu|
		model = Sketchup.active_model #this is the current sketchup model environemnt
		layers = model.layers #current set of layers
		current = model.active_layer #currently selected layer in layer palette
		currentname = current.name.to_s
		
		submenu=menu.add_submenu("Put on Layer: ")
			submenu.add_item("Current - " + currentname) { putCurrentLayer current } 
			if layers.count > 1 then #add layer options if there are more than 1 layer in model
				submenu.add_separator
				layers.each do |layer|
					submenu.add_item(layer.name) { putCurrentLayer layer } #add entries for each layer. call "putCurrentLayer" function by passing the layer you want to put everything on...
				end			
			end
	end
end
file_loaded("PutOnLayer.rb")